@@ -21,13 +21,14 @@ import time
 import typing
 
 from .config import ConfigObject
-from .dirinfo import DirInfo
 from .fileman import FileManager
+from .filescanner import FileScanner
 from .logman import LogManager
 from .transferlists import TransferLists
 from .utils import (
     getString,
     getVersion,
+    simplePrompt,
     readJson,
     writeJson,
     testConsistency
@@ -64,13 +65,27 @@ class BackupManager():
         # scan only mode
         if self.config.scan_only and (self.config.dest is None or not os.path.isdir(self.config.dest)):
             self.config.dest = self.config.source
+        # cold storage mode
+        if self.config.use_cold_storage:
+            self.config.write_database_x2 = True
         # check source & dest
         if not os.path.isdir(self.config.source):
-            print(self.log.colourString(getString("Invalid source directory: ") + self.config.source, "R"))
-            sys.exit()
+            print(self.log.colourString(getString("Unable to access source directory: ") + self.config.source, "R"))
+            print(self.log.colourString(getString("Check folder permissions and if the directory exists."), "R"))
+            sys.exit(1)
         if self.config.dest is None:
             print(self.log.colourString(getString("Destination directory not provided or config failed to load"), "R"))
-            sys.exit()
+            sys.exit(1)
+        try:
+            for access_test in [self.config.source, self.config.dest]:
+                if os.path.isdir(access_test):
+                    _ = os.listdir(access_test)
+                else:
+                    os.makedirs(access_test)
+        except Exception as e:
+            self.log.colourPrint("%s: %s for %s" % (type(e).__name__, str(e.args), access_test), "R")
+            self.log.colourPrint(getString("BackuPy will now exit without taking any action."), "R")
+            sys.exit(1)
         self.config.source = os.path.abspath(self.config.source)
         self.config.dest = os.path.abspath(self.config.dest)
         # save config (still works with --load, so you can cleanup a messy json file from an old version this way)
@@ -96,7 +111,7 @@ class BackupManager():
     def saveConfig(self) -> None:
         writeJson(os.path.join(self.config.source, self.config.config_dir, "config.json"), vars(self.config))
         print(self.log.colourString(getString("Config saved"), "G"))
-        sys.exit()
+        sys.exit(0)
 
     def loadConfig(self) -> None:
         # will probably raise an exception and crash (safely) if the configuration is not valid json
@@ -108,7 +123,7 @@ class BackupManager():
         print(self.log.colourString(getString("Loaded config from:") + "\n" + config_dir, "G"))
         if self.config.source is None or os.path.abspath(current_source) != os.path.abspath(self.config.source):
             print(self.log.colourString(getString("A config file matching the specified source was not found (case sensitive)"), "R"))
-            sys.exit()
+            sys.exit(1)
 
     def abortRun(self) -> int:
         self.log.append([getString("### ABORTED ###")])
@@ -125,7 +140,7 @@ class BackupManager():
         # print database conflicts, including both collisions from files being modified independently on both sides and unexpected missing files
         # note: this only notifies the user so they can intervene, it does not handle them in any special way, treating them as regular file changes
         # it can also be triggered by time zone or dst changes, lower file system mod time precision, and corruption if using CRCs (handled next)
-        abort_run = False
+        detection_flag = False
         if dest_database_load_success and self.config.source != self.config.dest:
             self.log.append([getString("### DATABASE CONFLICTS ###")], ["Section"])
             if self.config.main_mode == "sync":
@@ -134,7 +149,7 @@ class BackupManager():
                 sync_conflicts = sorted(list(filter(lambda f: not self.source.fileMatch(f, f, dest_dict, set(), True), sync_conflicts)))  # don't already match
                 if len(sync_conflicts) >= 1:
                     print(self.log.colourString(getString("WARNING: found files modified in both source and destination since last scan"), "Y"))
-                    abort_run = True
+                    detection_flag = True
                 print(self.log.colourString(getString("Sync Database Conflicts: %s") % (len(sync_conflicts)), "V"))
                 self.log.printSyncDbConflicts(sync_conflicts, source_dict, dest_dict, source_prev, dest_prev)
             else:
@@ -143,7 +158,7 @@ class BackupManager():
                 dest_conflicts += sorted(list(dest_new))
                 if len(dest_conflicts) >= 1:
                     print(self.log.colourString(getString("WARNING: found files modified in the destination since last scan"), "Y"))
-                    abort_run = True
+                    detection_flag = True
                 print(self.log.colourString(getString("Destination Database Conflicts: %s") % (len(dest_conflicts)), "V"))
                 self.log.printChangedFiles(dest_conflicts, dest_dict, dest_prev, "   Dest", "     DB")
         # print database conflicts concerning CRCs if available, as well as CRC conflicts between source and dest if attributes otherwise match
@@ -151,7 +166,7 @@ class BackupManager():
         if len(source_crc_errors) > 0 or len(dest_crc_errors) > 0:
             self.log.append([getString("### CRC ERRORS DETECTED ###")], ["Section"])
             print(self.log.colourString(getString("WARNING: found non matching CRC values, possible corruption detected"), "Y"))
-            abort_run = True
+            detection_flag = True
             crc_errors_detected = sorted(list(source_crc_errors | dest_crc_errors))
             print(self.log.colourString(getString("CRC Errors Detected: %s") % (len(crc_errors_detected)), "V"))
             if self.config.source != self.config.dest:
@@ -159,23 +174,26 @@ class BackupManager():
             else:
                 self.log.printChangedFiles(crc_errors_detected, source_dict, source_prev, " Source", "     DB")
         # show curses tree
-        if self.config.curses:
-            try:
-                from .treeman import dest_conflicts_tree, sync_conflicts_tree
-                self.log.writeLog("database.tmp.json")
-                if dest_database_load_success and self.config.source != self.config.dest:
-                    if self.config.main_mode == "sync":
-                        sync_conflicts_tree(sync_conflicts, crc_errors_detected)
+        if detection_flag:
+            while not self.config.noprompt:
+                print(self.log.colourString(getString("Show files as tree with curses (y/n)?"), "G"))
+                response = simplePrompt(["y", "n"])
+                if response == "n":
+                    break
+                try:
+                    from .treedisplay import dest_conflicts_tree, sync_conflicts_tree
+                    if dest_database_load_success and self.config.source != self.config.dest:
+                        if self.config.main_mode == "sync":
+                            sync_conflicts_tree(sync_conflicts, crc_errors_detected)
+                        else:
+                            dest_conflicts_tree(dest_new, dest_modified, dest_missing, crc_errors_detected)
                     else:
-                        dest_conflicts_tree(dest_new, dest_modified, dest_missing, crc_errors_detected)
-                else:
-                    sync_conflicts_tree([], crc_errors_detected)
-            except Exception:
-                print(self.log.colourString(getString("Curses Error"), "R"))
-                abort_run = True
-        return abort_run
+                        sync_conflicts_tree([], crc_errors_detected)
+                except Exception:
+                    print(self.log.colourString(getString("Curses Error"), "R"))
+        return detection_flag
 
-    def _printAndLogScanOnlyDiffSummary(self, side_str: str, side_info: DirInfo) -> None:
+    def _printAndLogScanOnlyDiffSummary(self, side_str: str, side_info: FileScanner) -> None:
         # get databases
         side_dict, side_prev = side_info.getDicts()
         side_new, side_modified, side_missing, side_crc_errors, side_dirs, side_unmodified = side_info.getSets()
@@ -199,13 +217,17 @@ class BackupManager():
         self.log.append([getString("### %s MOVED FILES ###") % (side_str.upper())], ["Section"])
         self.log.printMovedFiles(moved, side_dict, side_prev, "   New: ", "   Old: ")
         # show curses tree
-        if self.config.curses:
-            try:
-                from .treeman import scan_only_tree
-                self.log.writeLog("database.tmp.json")
-                scan_only_tree(side_str, list_new, list_missing, list_modified, moved)
-            except Exception:
-                print(self.log.colourString(getString("Curses Error"), "R"))
+        if list_new or list_missing or list_modified or moved:
+            while not self.config.noprompt:
+                print(self.log.colourString(getString("Show files as tree with curses (y/n)?"), "G"))
+                response = simplePrompt(["y", "n"])
+                if response == "n":
+                    break
+                try:
+                    from .treedisplay import scan_only_tree
+                    scan_only_tree(side_str, list_new, list_missing, list_modified, moved)
+                except Exception:
+                    print(self.log.colourString(getString("Curses Error"), "R"))
 
     def _printAndLogCompareDiffSummary(self, transfer_lists: TransferLists) -> None:
         # get lists and databases
@@ -297,30 +319,36 @@ class BackupManager():
             print(self.log.colourString(getString("Dry Run"), "V"))
         else:
             simulation_msg = ""
-        # init DirInfo and load previous scan data if available
-        self.source = DirInfo(self.config.source, self.config.source_unique_id,
+        # init FileScanner and load previous scan data if available
+        self.source = FileScanner(self.config.source, self.config.source_unique_id,
                               self.config.dest, self.config, self.gui)
-        self.dest = DirInfo(self.config.dest, self.config.dest_unique_id,
+        self.dest = FileScanner(self.config.dest, self.config.dest_unique_id,
                             self.config.source, self.config, self.gui)
         dest_database_load_success = False
         self.source.loadDatabase()
-        self.dest.loadDatabase()
+        self.dest.loadDatabase(self.config.use_cold_storage)
         if self.dest.dict_prev != {}:
             dest_database_load_success = True
         # scan directories (also calculates CRC if enabled) (didn't parallelize scans to prevent excess vibration of adjacent consumer grade disks and keep status bars simple)
-        self.log.colourPrint(getString("Scanning files on source:\n%s") % (self.config.source), "B")
-        self.source.scanDir(self.config.stdout_status_bar)
-        if self.config.source != self.config.dest:
-            self.log.colourPrint(getString("Scanning files on destination:\n%s") % (self.config.dest), "B")
-            self.dest.scanDir(self.config.stdout_status_bar)
-        else:
-            self.dest = self.source
+        try:
+            self.log.colourPrint(getString("Scanning files on source:\n%s") % (self.config.source), "B")
+            self.source.scanDir(self.config.stdout_status_bar)
+            if not self.config.use_cold_storage:
+                if self.config.source != self.config.dest:
+                    self.log.colourPrint(getString("Scanning files on destination:\n%s") % (self.config.dest), "B")
+                    self.dest.scanDir(self.config.stdout_status_bar)
+                else:
+                    self.dest = self.source
+        except Exception as e:
+            self.log.colourPrint(getString("Error encountered during scan: ") + str(e.args[0]), "R")
+            self.log.colourPrint(getString("BackuPy will now exit without taking any action."), "R")
+            return 1
         # update log manager to reference the same source and dest
         self.log.source, self.log.dest = self.source, self.dest
         # compare directories (should be relatively fast, all the read operations are done during scan)
         if not self.config.scan_only:
             self.log.colourPrint(getString("Comparing directories..."), "B")
-            transfer_lists = TransferLists(self.source.compareDirInfo(self.dest, self.config.nomoves))
+            transfer_lists = TransferLists(self.source.compareOtherScanner(self.dest, self.config.nomoves))
             if self.config.main_mode == "sync":
                 transfer_lists.updateSyncMovedDirection(self.dest)
                 if self.config.sync_propagate_deletions:
@@ -362,21 +390,20 @@ class BackupManager():
             if self.gui:
                 go = self.gui_simplePrompt(getString("Scan complete, continue with %s%s?") % (self.config.main_mode, simulation_msg))
             else:
-                print(self.log.colourString(getString("Scan complete, continue with %s%s (y/N/skip)?") % (self.config.main_mode, simulation_msg), "G"))
-                go = input("> ")
-            if len(go.strip()) == 4 and go.strip().lower() == "skip":
+                print(self.log.colourString(getString("Scan complete, continue with %s%s (y/n/skip/curses)?") % (self.config.main_mode, simulation_msg), "G"))
+                go = simplePrompt(["y", "n", "skip", "curses"])
+            if go == "skip":
                 if not transfer_lists.skipFileTransfers(self.log):
                     return self.abortRun()
-            elif len(go.strip()) == 6 and go.strip().lower() == "curses":
+            elif go == "curses":
                 try:
-                    from .treeman import transfer_lists_tree
+                    from .treedisplay import transfer_lists_tree
                     transfer_lists_tree(transfer_lists.getLists())
                 except Exception:
                     print(self.log.colourString(getString("Curses Error"), "R"))
-                    return self.abortRun()
-            elif len(go.strip()) == 0 or go.strip()[0].lower() != "y":
+            elif go == "n":
                 return self.abortRun()
-            elif go.strip()[0].lower() == "y":
+            elif go == "y":
                 break
         # backup operations
         self._performBackup(transfer_lists, simulation_msg)
