@@ -34,11 +34,11 @@ class ConfigObject:
         self.nolog = False
         self.nomoves = False
         self.noprompt = False
-        self.curses = False
         self.dry_run = False
         self.force_posix_path_sep = False
         self.quit_on_db_conflict = False
         self.scan_only = False
+        self.use_cold_storage = False
         self.use_rsync = False
         self.verify_copy = False
         # default config (additional)
